import {
  Column,
  Entity,
  JoinColumn,
  ManyToOne,
  OneToMany,
  PrimaryGeneratedColumn,
} from 'typeorm';
import { Equipo } from '../../equipo/entity/equipo.entity';
import { Multa } from '../../multa/entity/multa.entity';
import { Operador } from '../../operador/entity/operador.entity';
import { Usuario } from '../../usuario/entity/usuario.entity';

@Entity()
export class Prestamo {
  @PrimaryGeneratedColumn()
  id_prestamo: number;

  @Column({ type: Boolean, nullable: false, default: true })
  activo: boolean;

  @Column({ type: Boolean, nullable: false, default: false })
  cancelado_operador: boolean;

  @Column({ type: Boolean, nullable: false, default: false })
  cancelado_repeticion: boolean;

  @Column({ type: Boolean, nullable: false, default: false })
  cancelado_usuario: boolean;

  @Column({ type: Date, nullable: true, default: null })
  fecha_entrega: Date;

  @Column({ type: Date, nullable: false })
  fecha_inicio: Date;

  @Column({ type: Date, nullable: true, default: null })
  hora_fin: Date;

  @Column({ type: Date, nullable: true, default: null })
  hora_inicio: Date;

  @Column({ type: Date, nullable: false })
  hora_max_recoger: Date;

  @Column({ type: Number, nullable: true })
  id_equipo: number;

  @Column({ type: Number, nullable: true })
  id_operador_entrega: number;

  @Column({ type: Number, nullable: true })
  id_operador_regreso: number;

  @Column({ type: Number, nullable: true })
  id_usuario: number;

  @ManyToOne(() => Equipo, (equipo) => equipo.prestamos, { eager: true })
  @JoinColumn({ name: 'id_equipo' })
  equipo: Equipo;

  @ManyToOne(
    () => Operador,
    (operadorEntrega) => operadorEntrega.prestamosOperadorEntrega,
    { eager: true },
  )
  @JoinColumn({ name: 'id_operador_entrega' })
  operadorEntrega: Operador;

  @ManyToOne(
    () => Operador,
    (operadorRegreso) => operadorRegreso.prestamosOperadorRegreso,
    { eager: true },
  )
  @JoinColumn({ name: 'id_operador_regreso' })
  operadorRegreso: Operador;

  @ManyToOne(() => Usuario, (usuario) => usuario.prestamos, { eager: true })
  @JoinColumn({ name: 'id_usuario' })
  usuario: Usuario;

  @OneToMany(() => Multa, (multa) => multa.prestamo)
  multa: Multa;
}
